; -----------------------------------------------------------------------
; Šablona zdrojového kódu generovná programem Piklab
; -----------------------------------------------------------------------
    #include <p12f629.inc>

; -----------------------------------------------------------------------
; Konfigurační bity: upravte dle vašeho nastavení a potřeb
; -----------------------------------------------------------------------
    __CONFIG _INTRC_OSC_NOCLKOUT & _WDT_ON & _PWRTE_OFF & _MCLRE_OFF & _BODEN_ON & _CP_OFF & _CPD_OFF

; -----------------------------------------------------------------------
; makra
; -----------------------------------------------------------------------
#define	step_count	04h	; 4 kroky 0..3, 8 kroku 0..7
#define	step_dur	0FFh	; doba trvání kroku - cca 0.5 * step_dur [ms]
#define sample_preload	56h	; nastavení vzorkovací frekvence - delay = 3*sample_preload + 5 [us]
#define IR_INPUT	GPIO,3
#define rec_size	01h	; počet bytů IR příkazu
#define stor_fwd	0			; adresa EEPROM s povelem forward
#define stor_bck	stor_fwd+rec_size	; adresa EEPROM s povelem back

#define mode_learn_fwd	flags,0
#define mode_learn_bck	flags,1

#define	learn_count	5	; počet opakování při učení povelu 

#define select_bank1	bsf	STATUS,RP0  	;Bank 1
#define select_bank0	bcf	STATUS,RP0  	;Bank 0

; -----------------------------------------------------------------------
; Deklarace proměnných
; -----------------------------------------------------------------------
INT_VAR		UDATA 0x20
flags		RES 1 ; ruzne flagy
w_saved      	RES 1 ; variable used for context saving
status_saved 	RES 1 ; variable used for context saving

aux		RES 1 ; pomocny registr

step_cnt     	RES 1 ; pocitadlo kroku	
step_delay1	RES 1 ; casova prodleva
step_delay2	RES 1 ; casova prodleva

sample_delay	RES 1 ; prodleva mezi vzorky IR signálu

input_wait1	RES 1 ; čekání na klid na IR vstupu
input_wait2	RES 1 ;

byte_cnt	RES 1 ; pocitadlo bytu IR signalu
learn_repeat	RES 1 ; pocitadlo pokusů při učení povelu

rec_ptr_new	RES 1 ; ukazatele na buffery přijatých dat
rec_ptr_prev	RES 1 ;

rec_data1	RES rec_size ; přijatá data
rec_data2	RES rec_size ; data přijatá při předchozím pokusu při učení

; -----------------------------------------------------------------------
; vektor resetu
; -----------------------------------------------------------------------
STARTUP CODE 0x000
    goto    init           ; skok na začátek hlavního kódu

; -----------------------------------------------------------------------
; vektor přerušení
; -----------------------------------------------------------------------
INT_VECTOR CODE 0x004
    goto    interrupt      ; skok na začátek kódu pro obsluhu přerušení

; -----------------------------------------------------------------------
; přemístitelný kód
; -----------------------------------------------------------------------
PROG CODE

; -----------------------------------------------------------------------
; proved krok motoru
; -----------------------------------------------------------------------
STEP
	movf	step_cnt, w
;	call	STEP_half
	call	STEP_full
	movwf	GPIO

	movlw	step_dur
	movwf	step_delay2

	return

STEP_full
	addwf	PCL, f
	dt	10h, 20h, 04h, 01h

STEP_half
	addwf	PCL, f
	dt	30h, 20h, 24h, 04h, 05h, 01h, 11h, 10h

; -----------------------------------------------------------------------
; proved krok vpred
; -----------------------------------------------------------------------
STEP_FWD
	movf	step_cnt, w
	btfsc	STATUS, Z
	movlw	step_count
	movwf	step_cnt
	decf	step_cnt, f
	call	STEP
	return

; -----------------------------------------------------------------------
; proved krok vzad
; -----------------------------------------------------------------------
STEP_BCK
	incf	step_cnt, f
	movf	step_cnt, w
	xorlw	step_count
	btfsc	STATUS, Z
	clrf	step_cnt
	call	STEP
	return

; -----------------------------------------------------------------------
; zabzuč motorem - vyšší ton
; -----------------------------------------------------------------------
BEEP
	movlw	00h
	movwf	step_delay2

BEEP_loop
	movlw	05h
	movwf	GPIO

	movlw	80h
	movwf	step_delay1
BEEP_pulse_a
	decfsz	step_delay1, f
	goto	BEEP_pulse_a	
	movlw	30h
	movwf	GPIO

	movlw	80h
	movwf	step_delay1
BEEP_pulse_b
	decfsz	step_delay1, f
	goto	BEEP_pulse_b	

	decfsz	step_delay2, f
	goto	BEEP_loop	

	clrf	GPIO

	return

; -----------------------------------------------------------------------
; zabzuč motorem - nižší ton
; -----------------------------------------------------------------------
BEEP_LOW
	movlw	80h
	movwf	step_delay2

BEEP_LOW_loop
	movlw	05h
	movwf	GPIO

	movlw	00h
	movwf	step_delay1
BEEP_LOW_pulse_a
	decfsz	step_delay1, f
	goto	BEEP_LOW_pulse_a	
	movlw	30h
	movwf	GPIO

	movlw	00h
	movwf	step_delay1
BEEP_LOW_pulse_b
	decfsz	step_delay1, f
	goto	BEEP_LOW_pulse_b	

	decfsz	step_delay2, f
	goto	BEEP_LOW_loop	

	clrf	GPIO

	return

; -----------------------------------------------------------------------
; časová prodleva mezi vzorky IR signálu = 1/4 bitu:
; -----------------------------------------------------------------------
SAMPLE_DELAY
	movlw	sample_preload
	movwf	sample_delay

SAMPLE_DELAY_loop
	decfsz	sample_delay, f
	goto	SAMPLE_DELAY_loop
	
	return

; -----------------------------------------------------------------------
; porovnej přijatý povel v bufferu <- rec_ptr_new s povelem v EEPROM s adresou ve w:
; -----------------------------------------------------------------------
EVAL_CMD
	select_bank1
	movwf	EEADR

	movf	rec_ptr_new, w
	movwf	FSR

	movlw	rec_size
	movwf	byte_cnt

EVAL_CMD_loop
	bsf	EECON1, RD
	movf	INDF, w
	xorwf	EEDATA, w
	btfss	STATUS, Z	
	goto	EVAL_CMD_nomatch
	incf	FSR, f
	incf	EEADR, f		

	decfsz	byte_cnt, f
	goto	EVAL_CMD_loop

	select_bank0
	retlw	0		; OK

EVAL_CMD_nomatch
	select_bank0
	retlw	1		; ERROR

; -----------------------------------------------------------------------
; nastav flagy pro příslušný mod podle vstupu:
; -----------------------------------------------------------------------
READ_MODE
	bcf	mode_learn_fwd
	bcf	mode_learn_bck
	
	movlw	80h		; dolní práh
	select_bank1
	movwf	VRCON
	select_bank0
	call	READ_MODE_delay
	call	READ_MODE_delay
	btfsc	CMCON, COUT
	bsf	mode_learn_fwd

	movlw	8Fh		; horní práh
	select_bank1
	movwf	VRCON
	select_bank0
	call	READ_MODE_delay
	call	READ_MODE_delay
	btfss	CMCON, COUT
	bsf	mode_learn_bck

	return

READ_MODE_delay
	nop
	nop
	return

; -----------------------------------------------------------------------
; celá sranda se odehrává v přerušení:
; -----------------------------------------------------------------------
interrupt
	movlw	rec_size
	movwf	byte_cnt
	
	movf	rec_ptr_new, w
	movwf	FSR

	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY

	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY

	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY

	call	SAMPLE_DELAY

interrupt_recv_loop_byte
	movlw	1		; inicializuj registr - až tahle jednotka vyteče, ukonči cyklus
	movwf	INDF

interrupt_recv_loop_bit
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY
	call	SAMPLE_DELAY

	;bsf	GPIO, 5		;DEBUG

	bcf	STATUS, C
	btfsc	IR_INPUT	; přenes hodnotu vstupu do C
	bsf	STATUS, C
	rlf	INDF, f		; odrotuj registr

	;bcf	GPIO, 5		;DEBUG

	btfss	STATUS, C
	goto	interrupt_recv_loop_bit

	incf	FSR, f		; další byte...	

	decfsz	byte_cnt, f
	goto	interrupt_recv_loop_byte

; co ted s přijatými daty?
	call	READ_MODE

	btfsc	mode_learn_fwd
	goto	interrupt_learn
	btfsc	mode_learn_bck
	goto	interrupt_learn

; defaultní režim - zpracování přijatého povelu:
interrupt_eval_cmd

; je povel forward? :
	movlw	stor_fwd
	call	EVAL_CMD
	iorlw	0
	btfsc	STATUS, Z
	call	STEP_FWD
; je povel back? :
	movlw	stor_bck
	call	EVAL_CMD
	iorlw	0
	btfsc	STATUS, Z
	call	STEP_BCK

	goto	interrupt_exit

interrupt_learn

; v poli rec_data máme přijatá data, porovnáme je s daty v rec_prev:
	clrf	byte_cnt

interrupt_learn_compare
	movf	rec_ptr_new, w
	addwf	byte_cnt, w
	movwf	FSR
	movf	INDF, w
	movwf	aux

	movf	rec_ptr_prev, w
	addwf	byte_cnt, w
	movwf	FSR
	
	movf	INDF, w
	xorwf	aux, w
	btfss	STATUS, Z
	goto	interrupt_learn_error

	incf	byte_cnt, f
	movf	byte_cnt, w
	xorlw	rec_size
	btfss	STATUS, Z
	goto	interrupt_learn_compare

	;pokud jsme tady, porovnání s předchozím povelem dopadlo dobře	
	
	decfsz	learn_repeat, f
	goto	interrupt_exit	; jdi na dalsi pokus

	;pokud jsme tady, porovnání dopadlo dobře learn_repeat-krát	

interrupt_write
; uložíme data z bufferu do EEPROM:
	movf	rec_ptr_new, w
	movwf	FSR		; do FSR adresa pole rec_data

	movlw	rec_size
	movwf	byte_cnt	; počítadlo bytů

	movlw	stor_fwd	; default, pro jistotu...
	;btfsc	mode_learn_fwd
	;movlw	stor_fwd
	btfsc	mode_learn_bck
	movlw	stor_bck

	select_bank1

	movwf	EEADR		; do EEADR adresa, kam má být uložen povel
	bsf   	EECON1,WREN 	;Enable write

interrupt_write_loop
	movf	INDF, w
	movwf	EEDATA

	movlw 	0x55         	;Unlock write
	movwf 	EECON2      	;
	movlw 	0xAA         	;
	movwf 	EECON2      	;
	bsf   	EECON1,WR   	;Start the write

interrupt_write_wait
	btfsc	EECON1,WR	; počkej na dokončení zápisu
	goto	interrupt_write_wait

	incf	FSR, f		; další byte...	
	incf	EEADR, f

	decfsz	byte_cnt, f
	goto	interrupt_write_loop

	bcf   	EECON1,WREN 	;Disable write
	select_bank0
	bcf	PIR1, EEIF	; zruž flag přerušení konce zápisu

interrupt_write_beep
	; konec zápisu, zatrub :-)
	call	BEEP_LOW
	call	BEEP
	goto	interrupt_exit

	
interrupt_learn_error
; předpokládám, že první porovnání se nikdy nepovede,
; proto inicializaci learn_repeat dělám až tady
	movlw	learn_count
	movwf	learn_repeat
; prohod ukazatele na nový a předchozí buffer
	movf	rec_ptr_new, w
	movwf	aux
	movf	rec_ptr_prev, w
	movwf	rec_ptr_new
	movf	aux, w
	movwf	rec_ptr_prev

	call	BEEP

	goto	interrupt_exit

interrupt_mode

interrupt_exit

; počkáme na klid na IR vstupu - cca 5 ms:
	clrf	input_wait1
	movlw	05h
	movwf	input_wait2
interrupt_exit_wait
	btfss	IR_INPUT
	goto	interrupt_exit
	decfsz	input_wait1, f
	goto	interrupt_exit_wait
	decfsz	input_wait2, f
	goto	interrupt_exit_wait
	
	bcf	INTCON, GPIF	; nuluj priznak preruseni od vstupu
	
	retfie


; -----------------------------------------------------------------------
; inicializace procesoru:
; -----------------------------------------------------------------------
init
	select_bank1

	call  	3FFh        	;Get the cal value
	movwf 	OSCCAL      	;Calibrate
	
	movlw	0Ah
	movwf	TRISIO		; GPIO 0,2,4,5 vystupy

	movlw	08h
	movwf	IOC		; vstup 3 aktivuje přerušení

	bsf	INTCON, GPIE	; povol preruseni od zmeny vstupu

	movlw	0xFF
	movwf	OPTION_REG	; preddelicka 1:128 pro WDT ~ cca 2 s

	select_bank0

	movlw	04h
	movwf	CMCON		; nastav komparator - vnitrni reference

	movlw	1
	movwf	step_cnt

	movlw	rec_data1	; napln ukazatele na buffery
	movwf	rec_ptr_new

	movlw	rec_data2
	movwf	rec_ptr_prev

	clrf	GPIO

	bsf	INTCON, GIE	; povol preruseni globalne

	;bsf	mode_learn_fwd

; -----------------------------------------------------------------------
; hlavní smyčka:
; -----------------------------------------------------------------------
run

STEP_delay
	clrwdt
	decfsz	step_delay1, f
	goto	STEP_delay
	decfsz	step_delay2, f
	goto	STEP_delay

	clrf	GPIO

	goto    run              ; opakuj navždy

END
